#!/bin/sh

# Print the intro
echo '
This is the configuration tool of GPMI. It will set up all the Makefiles
and other generated files so you can configure and install GPMI. Basicly
it works in two steps: collect all information then build files. Information
is cullected in variables. Variables get their values from these sources
(later mentioned source may override earlier ones):
  1. config/configure.defaults (do not edit)
  2. config/local.conf (you may create this file in the style of 
	   file config/configure.defaults)
  3. command line arguments (--with-*=something and --without-*)
	4. one of the shell scripts in config/os/ (system specific settings);
	   should not override previous settings but use them as base
	5. autodetected values; should be run only for variables not yet set

List of parameters which are common for all platforms:
'

# Process configure.defaults
awk '
	BEGIN {
		FS="[ 	=]+"
		R=0
	}

	function fillup(str, len) {
		while(length(str) < len) str=str " "
		return str
	}

	/Rwith_/ {
		if (R == 0) {
			sub("R", "", $1)
			print " " $1
			R = 1
		}
		next
	}


	/with_/ {
		print " " fillup($1, 30) last
		R = 0
	}
	
	/^#/ {
		last = $0
		sub("^#.*-- ", "", last)
	}
' < configure.defaults | sort


echo '
List of platform specific parameters:
'

cat os_help/*
