#ifndef inc_hash_h
#define inc_hash_h
typedef struct hdata_t {
	void *key;
	void *data;
	struct hdata_t *next;
} hdata_t;

typedef struct hash_t {
	struct hdata_t **tbl;
	int tbl_size;
	void *hash_calc;
} hash_t;

enum hash_commands {
	h2_cmd_key      = 1,		//calc key from string 1
	h2_cmd_cmpkey   = 2,		//return 0 if keys matching
	h2_cmd_cmprec   = 3,		//return 0 if records matching
	h2_cmd_allockey	= 4,		//allocates, clones str and returns a pointer to the new key space
	h2_cmd_freekey	= 5,		//frees key made using h2_cmd_allockey
	h2_cmd_destroy	= 6,		//called when a record is destroyed - useful for freeing data
} ;


void hash_store(struct hash_t *h, const void *key, const void *data);	//! store a data-key pair


struct hdata_t *hash_find_(struct hash_t *h, const void *key);	//!find a key, returns a hash record

/* find a key, returns data */
const void *hash_find(struct hash_t *h, const void *key);
const void *hash_find_next(struct hash_t *h, const void *key, struct hdata_t **a);
const void *hash_find_first(struct hash_t *h, const void *key, struct hdata_t **a);


struct hash_t *hash_create(int tbl_size, void *calc);	//!create a table


int hash_stat(struct hash_t *h, int *recno, int *empty_nodes, float *avg_rec_per_node, int *max_rec_per_node, int *memory);
//!< statistics on the hash table


struct hash_t *hash_rehash(struct hash_t *h, int new_size, void *new_calc);
//!< reconstruct the hash table: resize and/or change calc algorithm

int hash_destroy(struct hash_t *h);
//!< destroys the hash table and frees up all the memory


unsigned int hash_defcalc(int cmd, char *str, char *str2, char **result_pointer);
//!< default hashing routine to use


int hash_del_key(struct hash_t *h, void *key);
//!< deletes a key; the caller has to free the data!


int hash_del_rec(struct hash_t *h, const void *key, const void *data);
//!< deletes a record; the caller has to free the data!

/* This macro provides a NULL-spoof strdup */
#define hash2_astrcpy(snew,sold)	if (sold!=NULL) snew = strdup(sold); \
														else snew=NULL;


#endif
