/*
    libgpmi - General Package/Module Interface
    Copyright (C) 2005-2007 Patric 'TrueLight' Stout & Tibor 'Igor2' Palinkas
  
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
  
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
  
    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
/**
 * @file gpmi/keywords.h
 * \brief Handles keywords for GPMI.
 *
 * Mostly used for script-modules.
 */

#ifndef GPMI_KEYWORDS_H
#define GPMI_KEYWORDS_H

/**
 * Structures for storing static keyword lists - intended to use in .c files
 * for static keyword lists.
 */
typedef struct gpmi_keyword_static_int {
	const char *name;
	int value;
} gpmi_keyword_static_int;

/**
 * Structures for storing static keyword lists - intended to use in .c files
 * for static keyword lists.
 */
typedef struct gpmi_keyword_static_string {
	const char *name;
	const char *value;
} gpmi_keyword_static_string;

/**
 * Structures for storing static keyword lists - intended to use in .c files
 * for static keyword lists.
 */
typedef struct gpmi_keyword_static_ptr {
	const char *name;
	void *value;
} gpmi_keyword_static_ptr;

typedef enum gpmi_keyword_types {
	gpmi_keyword_type_integer = 1,
	gpmi_keyword_type_string  = 2,
	gpmi_keyword_type_pointer = 3,
} gpmi_keyword_types;
//!< Possible types of keyword-lists.



/**
 * Keyword hash structure. Stores some general properties about a given
 * keyword hash table and the table itself.
 */
typedef struct gpmi_keyword {
	gpmi_keyword_types type;  //! value type (one of gpmi_keyword_types)
	char *name;               //! table name
	void *hash;               //! hash table of keyword-value pairs
	void *hash_rev;           //! hash table for reverse-lookup
	int end;                  //! for auto-increment lists
	int flags;
} gpmi_keyword;

enum gpmi_keyword_flags {
	gpmi_keyword_flag_malloced = 1,
};

/* Create / import a new table */
gpmi_keyword *gpmi_keyword_create(const char *table_name, gpmi_keyword_types value_type);
gpmi_keyword *gpmi_keyword_import(const char *table_name, gpmi_keyword_types value_type, const void *array);

/* Destroy an existing table */
void gpmi_keyword_destroy(gpmi_keyword *kw);

/* Routines to add data in keyword tables */
int gpmi_keyword_add(gpmi_keyword *keyword_table, const char *name, ...);
int gpmi_keyword_add_auto(gpmi_keyword *keyword_table, const char *name);

/* Find routines - one for each type */
int gpmi_keyword_find_int(gpmi_keyword *keyword_table, const char *name);
int gpmi_keyword_find_int_numeric(gpmi_keyword *keyword_table, const char *name);
const char *gpmi_keyword_find_string(gpmi_keyword *keyword_table, const char *name);
void *gpmi_keyword_find_ptr(gpmi_keyword *keyword_table, const char *name);
int gpmi_keyword_find_multi(gpmi_keyword *array, const char *key, int *result);

/* Find reverse - supply one value and get back it's name. Value type must match keyword_table's value type! */
const char *gpmi_keyword_find_rev(gpmi_keyword *keyword_table, ...);
const char *gpmi_keyword_find_rev_multi(gpmi_keyword *keyword_table, int bitfield);

/* Delete data */
int gpmi_keyword_del(gpmi_keyword *keyword_table, const char *name);

/* Misc functions */
void gpmi_keyword_init(void);
void gpmi_keyword_uninit(void);

#endif /* GPMI_KEYWORDS_H */
