/*
    libgpmi - General Package/Module Interface
    Copyright (C) 2005-2007 Patric 'TrueLight' Stout & Tibor 'Igor2' Palinkas
  
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
  
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
  
    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/* This file provides an infrastructure to register and find static modules. */

#include <stdlib.h>
#include "libs.h"
#include "libs-module.h"
#include "mods-static.h"
#include "threads.h"

/**
 * Check if a module is static, and if it is, set the correct pointers.
 */
int gpmi_load_static_module(gpmi_module *module, const char *library_name)
{
	void (*reg_proc)(gpmi_module *module);

	gpmi_threads_init();

	gpmi_shared_lock;
	if (gpmi_shared.static_modules == NULL) {
		gpmi_shared_unlock;
		return -2;
	}

	reg_proc = hash_find(gpmi_shared.static_modules, library_name);
	if (reg_proc == NULL) {
		gpmi_shared_unlock;
		return -1;
	}

	/* TODO -- Change this to load a library too */
	module->library = NULL;
	reg_proc(module);
	gpmi_shared_unlock;
	return 0;
}

/**
 * The constructor of a module calls this functions, letting us know
 *  that it is alive.
 */
void gpmi_reg_static_module(const char *library_name, void *reg_module)
{
	gpmi_threads_init();

	gpmi_shared_lock;
	if (gpmi_shared.static_modules == NULL)
		gpmi_shared.static_modules = hash_create(16, NULL);

	/* Add the create function to a hash so we can find it back later */
	hash_store(gpmi_shared.static_modules, library_name, reg_module);
	gpmi_shared.static_modules_count++;
	gpmi_shared_unlock;
}

/**
 * The destructor of a module calls this functions, letting us know
 *  that it is off.
 */
void gpmi_unreg_static_module(const char *library_name)
{

	/* Destroy the hash when all static modules are gone */
	gpmi_shared_lock;
	gpmi_shared.static_modules_count--;
	if (gpmi_shared.static_modules_count == 0) {
		hash_destroy(gpmi_shared.static_modules);
		gpmi_shared.static_modules = NULL;
		gpmi_uninit();
	}
	gpmi_shared_unlock;

}
