/*
    libgpmi - General Package/Module Interface - wrapgen
    Copyright (C) 2005-2007 Patric 'TrueLight' Stout & Tibor 'Igor2' Palinkas
  
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
  
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
  
    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include "wrapgen.h"
#include "generators.h"
#include "../gpmi/global.h"

/* All the files */
static str_buff_t	*wrp_events_c 		= NULL,
									*wrp_events_list	= NULL;



void GenPkgEventsFiles(int command)
{
	FILE *f;

	switch (command) {
		case open_files:
			wrp_events_c    = str_buff_open();
			wrp_events_list = str_buff_open();

			sbprintf(wrp_events_list, "\n");
			sbprintf(wrp_events_c,  "int %s%s_events_init() {\n", PREFIX_GPMI_PKG_GEN, name_space);
			break;
		case close_files:
			sbprintf(wrp_events_c, "\treturn 0;\n}\n\n");

			cfopen(f, gen_prefix "events.c", "w");
			str_buff_print(f, wrp_events_c);
			str_buff_print(f, wrp_events_list);
			cfclose(f);

			str_buff_close(wrp_events_c);
			str_buff_close(wrp_events_list);
			break;
	}
}


/* Wrapper generator for events */
void GenPkgEvents(void *current, int line, char *cmd, char *cmd_type, int cmd_type_flags, int params)
{
	char buff[1024], *s;
	int n;

	sbprintf(wrp_events_list,  "int %s = -1;\n", cmd);

	/* Empty input line */
	if ( cmd == NULL ) {
		sbprintf(wrp_events_c, "\n");
		return;
	}

	/* Generate spaces */
	for(n = 0; n <= 256; n++)
		buff[n] = ' ';
	buff[n] = 0;

	/* Generate param conversion */
	for (s = buff, n = 0; n < params; n++, s++) {
		switch (type_[n]) {
			case type_uint: 		*s = 'u'; break;
			case type_uchar: 		*s = 'C'; break;
			case type_char: 		*s = 'c'; break;
			case type_int: 			*s = 'i'; break;
			case type_double:		*s = 'd'; break;
			case type_string:		*s = 's'; break;
			case type_keyword:	*s = 'k'; break;
			case type_ptr:      *s = 'p'; break;
			default:
				fprintf(stderr, "Invalid type in GenPkgEvents: %d\n", type_[n]);
				*s = '#';
		}
	}
	*s = 0;

	sbprintf(wrp_events_c, "\t%s = gpmi_event_register(\"%s\", \"%s\");\n", cmd, cmd, buff);
}
