/*
    libgpmi - General Package/Module Interface - wrapgen
    Copyright (C) 2005-2007 Patric 'TrueLight' Stout & Tibor 'Igor2' Palinkas
  
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
  
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
  
    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include "wrapgen.h"
#include "wrapgen_keywords.h"
#include "generators.h"
#include "../gpmi/global.h"

/* All the files */
static str_buff_t	*wrp_functions_c = NULL,
		 							*pkgcreate_c     = NULL;



void GenPkgFuncFiles(int command)
{
	FILE *f;

	switch (command) {
		case open_files:
			wrp_functions_c = str_buff_open();
			pkgcreate_c     = str_buff_open();


			sbprintf(wrp_functions_c, "#include <stdlib.h>\n");
			sbprintf(wrp_functions_c, "#include <stdio.h>\n");
			sbprintf(wrp_functions_c, "#include <string.h>\n");
			sbprintf(wrp_functions_c, "#include <gpmi.h>\n");
			sbprintf(wrp_functions_c, "#include \"package.h\"\n");
			sbprintf(wrp_functions_c, "#include <gpmi/packages.h>\n");
			sbprintf(wrp_functions_c, "#include <gpmi/debug.h>\n\n");

			/* Write down the NAMESPACE check that is needed in packages */
			sbprintf(wrp_functions_c, "#ifndef NAMESPACE\n#error You should define NAMESPACE in Makefile.am\n#endif\n\n");
			sbprintf(pkgcreate_c,		 "void %s%s_register_commands_(void (*regcmd)(void *handle, char *name, void *proc_wrap, void *proc_lowlevel, void *package), void *handle, void *package) {\n", PREFIX_GPMI_PKG_GEN, name_space);

			break;
		case close_files:

			sbprintf(pkgcreate_c,		 "}\n");
			sbprintf(wrp_functions_c, "\n");

			cfopen(f, gen_prefix "functions.c", "w");
			str_buff_print(f, wrp_functions_c);
			str_buff_print(f, pkgcreate_c);
			cfclose(f);
			str_buff_close(wrp_functions_c);
			str_buff_close(pkgcreate_c);
			break;
	}
}

/* Wrapper generator for functions */
void GenPkgFunctions(void *current, int line, char *cmd, char *cmd_type, int cmd_type_flags, int params)
{
	char buff[1024];
	int n, longest_name, longest_type, var_count, kw_type;
	char *return_kw_type = "";

	/* Empty input line */
	if ( cmd == NULL ) {
		sbprintf(pkgcreate_c, "\n");
		return;
	}

	/* Generate spaces */
	for(n = 0; n <= 256; n++)
		buff[n] = ' ';
	buff[n] = 0;


	/* Create a pkg command */
	if ((cmd_type_flags & flag_direct) != 0) {
		sbprintf(pkgcreate_c, "\tregcmd(handle, \"%s\",%s%s,%s%s, package);\n", cmd, strlen(cmd) < 31 ? buff + 225 + strlen(cmd) : " ", cmd, strlen(cmd) < 31 ? buff + 225 + strlen(cmd) : " ", cmd);
		return;
	}
	else
		sbprintf(pkgcreate_c, "\tregcmd(handle, \"%s\",%sPkgW_%s,%s%s, package);\n", cmd, strlen(cmd) < 31 ? buff + 225 + strlen(cmd) : " ", cmd, strlen(cmd) < 31 ? buff + 225 + strlen(cmd) : " ", cmd);


	/* Init variables */
	longest_name = 0;
	longest_type = 0;
	var_count		 = 0;

	/* Generate the header */
	sbprintf(wrp_functions_c, "char *PkgW_%s(int argc, char *argv[])\n", cmd);
	sbprintf(wrp_functions_c, "{\n\tchar *Result;\n");

	/* Print the definition result_tmp, if needed */
	if ( (cmd_type_flags & flag_dynamic) != 0) {
		switch (FindType(cmd_type)) {
			case type_uchar:
			case type_char:
			case type_int:
			case type_void:
			case type_double:
			case type_keyword:
				fprintf(stderr,"Warning: can't make type %s dynamic for %s\n", cmd_type, cmd);
				break;
			case type_string:
//				sbprintf(wrp_functions_c, "\tchar *result_tmp;\n");
//				var_count++;
				break;
		}
	}

	/* Generate variable definitions */
	var_count += PrintVarDefs(&longest_name, &longest_type, params, cmd, wrp_functions_c);

	/* If we had variable definitions, print a separator newline */
	if ( var_count  != 0)
		sbprintf(wrp_functions_c, "\n");

	/* Generate param num check */
	sbprintf(wrp_functions_c, "\tPKG_Check_Argc(%d);\n", params);


	/* Generate param conversion */
	for (n = 0; n < params; n++) {
		switch (type_[n]) {
			case type_uchar:
			case type_char:
			case type_uint:
			case type_int:
				sbprintf(wrp_functions_c, "\tPKG_ConvertInteger(        %s,%s                      %s %d);\n",
					param[n], buff + 256 - longest_name + strlen(param[n]), buff + 256 - longest_type + 0, n + 1);
				break;
			case type_double:
				sbprintf(wrp_functions_c, "\tPKG_ConvertDouble(        %s,%s                      %s %d);\n",
					param[n], buff + 256 - longest_name + strlen(param[n]), buff + 256 - longest_type + 0, n + 1);
				break;
			case type_string:
				sbprintf(wrp_functions_c, "\tPKG_ConvertString(%s, %d);\n", param[n], n + 1);
				break;
			case type_ptr:
				sbprintf(wrp_functions_c, "\tPKG_ConvertPtr(%s, %d);\n", param[n], n + 1);
				break;
			case type_keyword:
				kw_type = keyword_find(type[n]);
				if ( flags[n] & flag_multiple ) {
					/* lookup for multiple keywords */
					if (kw_type != type_int) {
						fprintf(stderr, "Error: multiple keyword is allowed only for integer types!\n");
						exit(1);
					}
					sbprintf(wrp_functions_c, "\tif (gpmi_keyword_find_multi(%s, argv[%d], &%s) != 0) {\n", type[n], n + 1, param[n]);
					sbprintf(wrp_functions_c, "\t\tPKG_KeywordErrorAndReturn(%s, %d);\n\t}\n", param[n], n + 1);
				}
				else {
					/* lookup for single keyword */
					sbprintf(wrp_functions_c, "\t%s = gpmi_keyword_find_int(%s, argv[%d]);\n", param[n], type[n], n + 1);
					sbprintf(wrp_functions_c, "\tif (%s %s) {\n", param[n], inv_test[kw_type]);
					sbprintf(wrp_functions_c, "\t\tPKG_KeywordErrorAndReturn(%s, %d);\n\t}\n", param[n], n + 1);
				}
				break;
			default:
				fprintf(stderr, "ERROR: GenPkgFunctions can't handle type %s!\n", type_names[type_[n]]);
		}
	}

	/* Generate result part */
	{

		{
			if ( FindType(cmd_type) != type_keyword ) {
				cmd_type[0] = toupper(cmd_type[0]);
				sbprintf(wrp_functions_c, "\n\tPKG_Result_%s(", cmd_type);
			}
			else {
				if (cmd_type_flags & flag_multiple)
					return_kw_type = "_Multiple";
				else
					return_kw_type = "";
				sbprintf(wrp_functions_c, "\n\tPKG_Result_Keyword%s(%s, ", return_kw_type, cmd_type);
			}

			/* static version differs for strings: have to strdup */
			if (((cmd_type_flags & flag_dynamic) == 0 ) && ( FindType(cmd_type) == type_string))
				sbprintf(wrp_functions_c, "strdup_safe(");

			/* The actual interface call */
			sbprintf(wrp_functions_c, "%s(", cmd);
			for (n = 0; n < params; n++) {
				if ( n != 0 )
					sbprintf(wrp_functions_c, ", ");
				sbprintf(wrp_functions_c, "%s",param[n]);
			}

			/* static version differs for strings: have to close strdup */
			if (((cmd_type_flags & flag_dynamic) == 0 ) && ( FindType(cmd_type) == type_string))
				sbprintf(wrp_functions_c, ")");
		}

		/* close the call - static and dynamic differs again */
		if ((( cmd_type_flags & flag_dynamic ) == 0 ) || (FindType(cmd_type) == type_string)) {
			sbprintf(wrp_functions_c, "));\n");
		}
		else {
			sbprintf(wrp_functions_c, ");\n");
			if ( FindType(cmd_type) != type_keyword ) {
				sbprintf(wrp_functions_c, "\tPKG_Result_%s(result_tmp);\n", cmd_type);
			}
			else {
				// Do we ever get here?
				sbprintf(wrp_functions_c, "\tPKG_Result_Keyword%s(%s, result_tmp);\n", return_kw_type, cmd_type);
			}
			sbprintf(wrp_functions_c, "\tif ( result_tmp != NULL )\n\t\tfree(result_tmp);\n");
		}
	}
	sbprintf(wrp_functions_c, "\treturn Result;\n}\n\n");

}
